import argparse

from schrodinger import structure
from schrodinger.structutils import analyze, measure
from schrodinger.application import prepwizard
from schrodinger.protein import sequence as protein_sequence


from Bio import SeqIO
import Bio.SeqUtils as sequtils
from Bio.Seq import Seq
from Bio import pairwise2


def check_sequence_numbers(st):
    
    seqs = {}
    missing_res = {}
    
    
    output = ""

    for chain in st.chain:
                
        seq_list = []
        missing_residues = []
        
        for ind, res in enumerate(chain.residue):
            
        
            if res.isStandardResidue():
                            
                if ind !=0:
                    #enough to treat insertion codes?
                    if res.resnum != seq_list[-1]+1:
                        missing_residues+= [i for i in range(seq_list[-1]+1, res.resnum)]

                seq_list.append(res.resnum)

        seqs[chain.name] = seq_list
        missing_res[chain.name] = missing_residues
        
        
    for c, res in missing_res.items():
        
        if res == []:
            output += "MATCH for chain {}\n".format(c)
        else:
            output += "MISMATCH for chain {}\n".format(c)
            output += "Missing residues: {}\n\n".format(res)
                
    return output        
            
    
    
def get_structure_seq( st ):    
    
    sequences = {}
    for chain in st.chain:
        res_list = []

        for ind, res in enumerate(chain.residue):
            # ignore waters and ions 
            if res.isStandardResidue():
                res_list.append(res.getCode())
                
        sequences[chain.name] = res_list
        
    return sequences            

def parse_fasta( fasta_file):
    
    sequences = []
    for seq in SeqIO.parse( fasta_file , "fasta"):
        sequences.append(seq.seq)
    return sequences



def run_alignment(structure_seqs, fasta_seq, gap_open, gap_extend):
    
    if len(fasta_seq) > 1:
        print("Not yet implemented")
        exit(-1)
    
    struct_sequences = {}
    #convert structure_seqs to Biopython Sequences
    
    
    output = ""
    
    for c, seq in structure_seqs.items():
                
        s = "".join(seq) 
                
        # generate alignment
        ali = pairwise2.align.globalxs(s, fasta_seq[0],gap_open,gap_extend)
        ali_text = (pairwise2.format_alignment(*ali[0]))
        
        #parse alignment
        
        ali_split = ali_text.split("\n")
        struct_ali = ali_split[0]
        seq_ali = ali_split[2]
                
        #check if struct_ali contains any non-terminal gaps
        if "-" in struct_ali.strip("-") or "-" in seq_ali.strip("-"):
            output += "MISMATCH for chain {}\n".format(c)            
        else:
            output += "MATCH for chain {}\n".format(c)
        output += "Alignment:\n{}\n\n".format(ali_text)
        
    return output
            
    
def main():

    parser = argparse.ArgumentParser(
        description="Check if structure sequences have consistent residue numbering or, if given a fasta file, matche a given Uniprot sequence (Terminal gaps are tolerated).")
    
    parser.add_argument("infile", help = "input structure file")
    parser.add_argument("-f", "--fasta", help="fasta file")
    parser.add_argument("-o", "--output", help="output file")
    parser.add_argument("--gap_open", default = -2, type=float)
    parser.add_argument("--gap_extend", default = -0.1, type=float)
    
    args = parser.parse_args()
    
    
    #Currently only considers one structure entry
    st = next(structure.StructureReader(args.infile))
    
    
    if args.fasta:

        fasta_seqs = parse_fasta(args.fasta)
        structure_seqs = get_structure_seq( st )
        
        output = run_alignment(structure_seqs, fasta_seqs, args.gap_open, args.gap_extend)
    
    
    else:
        
        output = check_sequence_numbers( st )
        
    if not args.output:
        print(output)
        
        
    else:
        with open(args.output, "w") as outfile:
            outfile.write(output)
        



if __name__ == "__main__":
    main()